@@ -495,6 +495,13 @@ void BugzillaDuplicatesPage::performSearch()
 
     ui.m_statusWidget->setBusy(i18nc("@info:status","Searching for duplicates (from %1 to %2)...",
                                    startDateStr, endDateStr));
+                           
+    //Bugzilla will not search on Today bugs if we send the date.
+    //we need to send "Now"
+    if (m_searchingEndDate == QDate::currentDate()) {
+        endDateStr = QLatin1String("Now");
+    }
+    
     BugReport report = reportInfo()->newBugReportTemplate();
     bugzillaManager()->searchBugs(m_currentKeywords, report.product(), report.bugSeverity(),
                                     startDateStr, endDateStr, reportInfo()->firstBacktraceFunctions().join(" "));
@@ -808,7 +815,6 @@ BugzillaSendPage::BugzillaSendPage(DrKonqiBugReport * parent)
                             KIcon("document-preview"),
                             i18nc("@info:tooltip", "Use this button to show the generated "
                             "report information about this crash.")));
-    ui.m_showReportContentsButton->setVisible(false);
     connect(ui.m_showReportContentsButton, SIGNAL(clicked()), this, SLOT(openReportContents()));
                                                   
     ui.m_retryButton->setVisible(false);
@@ -822,7 +828,6 @@ BugzillaSendPage::BugzillaSendPage(DrKonqiBugReport * parent)
 
 void BugzillaSendPage::retryClicked()
 {
-    ui.m_showReportContentsButton->setVisible(false);
     ui.m_retryButton->setEnabled(false);
     aboutToShow();
 }
@@ -861,8 +866,6 @@ void BugzillaSendPage::sendError(QString errorString)
     ui.m_statusWidget->setIdle(i18nc("@info:status","Error sending the crash report:  "
                                   "<message>%1.</message>", errorString));
 
-    ui.m_showReportContentsButton->setVisible(true);
-    
     ui.m_retryButton->setEnabled(true);
     ui.m_retryButton->setVisible(true);
 }
