@@ -39,6 +39,8 @@
 #include <kpluginfactory.h>
 #include <kpluginloader.h>
 
+#include "kpasswdserveradaptor.h"
+
 K_PLUGIN_FACTORY(KPasswdServerFactory,
                  registerPlugin<KPasswdServer>();
     )
@@ -48,6 +50,12 @@ K_EXPORT_PLUGIN(KPasswdServerFactory("kpasswdserver"))
 #define AUTHINFO_EXTRAFIELD_ANONYMOUS "anonymous"
 #define AUTHINFO_EXTRAFIELD_BYPASS_CACHE_AND_KWALLET "bypass-cache-and-kwallet"
 
+qlonglong getRequestId()
+{
+    static qlonglong nextRequestId = 0;
+    return nextRequestId++;
+}
+
 int
 KPasswdServer::AuthInfoContainerList::compareItems(Q3PtrCollection::Item n1, Q3PtrCollection::Item n2)
 {
@@ -71,15 +79,33 @@ KPasswdServer::AuthInfoContainerList::compareItems(Q3PtrCollection::Item n1, Q3P
 KPasswdServer::KPasswdServer(QObject* parent, const QList<QVariant>&)
  : KDEDModule(parent)
 {
-    m_authPending.setAutoDelete(true);
+    KIO::AuthInfo::registerMetaTypes();
     m_seqNr = 0;
     m_wallet = 0;
+
+    KPasswdServerAdaptor *adaptor = new KPasswdServerAdaptor(this);
+    // register separately from kded
+    QDBusConnection::sessionBus().registerService("org.kde.kpasswdserver");
+    // connect signals to the adaptor
+    connect(this,
+            SIGNAL(checkAuthInfoAsyncResult(qlonglong, qlonglong, const KIO::AuthInfo &)),
+            adaptor,
+            SIGNAL(checkAuthInfoAsyncResult(qlonglong, qlonglong, const KIO::AuthInfo &)));
+    connect(this,
+            SIGNAL(queryAuthInfoAsyncResult(qlonglong, qlonglong, const KIO::AuthInfo &)),
+            adaptor,
+            SIGNAL(queryAuthInfoAsyncResult(qlonglong, qlonglong, const KIO::AuthInfo &)));
+    
     connect(this, SIGNAL(windowUnregistered(qlonglong)),
             this, SLOT(removeAuthForWindowId(qlonglong)));
 }
 
 KPasswdServer::~KPasswdServer()
 {
+    // TODO: what about clients waiting for requests? will they just
+    //       notice kpasswdserver is gone from the dbus?
+    qDeleteAll(m_authPending);
+    qDeleteAll(m_authWait);
     qDeleteAll(m_authDict);
     delete m_wallet;
 }
@@ -174,90 +200,147 @@ static bool readFromWallet( KWallet::Wallet* wallet, const QString& key, const Q
     return false;
 }
 
+bool KPasswdServer::hasPendingQuery(const QString &key, const KIO::AuthInfo &info)
+{
+    QString path2 = info.url.directory(KUrl::AppendTrailingSlash | KUrl::ObeyTrailingSlash);
+    Q_FOREACH(const Request *request, m_authPending) {
+        if (request->key != key) {
+            continue;
+        }
+
+        if (info.verifyPath) {
+            QString path1 = request->info.url.directory(KUrl::AppendTrailingSlash |
+                                                        KUrl::ObeyTrailingSlash);
+            if (!path2.startsWith(path1)) {
+                continue;
+            }
+        }
+
+        return true;
+    }
+
+    return false;
+}
+
 QByteArray
-KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, qlonglong usertime, const QDBusMessage &msg)
+KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, qlonglong usertime)
 {
     KIO::AuthInfo info;
-    {
-        QByteArray data2(data);
-        QDataStream stream(&data2, QIODevice::ReadOnly);
-        stream >> info;
-    }
+    QDataStream stream(data);
+    stream >> info;
     kDebug(130) << "User =" << info.username << ", WindowId =" << windowId << endl;
     if( usertime != 0 )
         kapp->updateUserTimestamp( usertime );
 
+    // if the check depends on a pending query, delay it
+    // until that query is finished.
     QString key = createCacheKey(info);
-
-    Request *request = m_authPending.first();
-    QString path2 = info.url.directory(KUrl::AppendTrailingSlash|KUrl::ObeyTrailingSlash);
-    for(; request; request = m_authPending.next())
-    {
-       if (request->key != key)
-           continue;
-
-       if (info.verifyPath)
-       {
-         QString path1 = request->info.url.directory(KUrl::AppendTrailingSlash|KUrl::ObeyTrailingSlash);
-          if (!path2.startsWith(path1))
-             continue;
-       }
-
-       msg.setDelayedReply(true);
-       request = new Request;
-       request->transaction = msg;
-       request->key = key;
-       request->info = info;
-       m_authWait.append(request);
-       return data;             // return value will be ignored
+    if (hasPendingQuery(key, info)) {
+        setDelayedReply(true);
+        Request *pendingCheck = new Request;
+        pendingCheck->isAsync = false;
+        pendingCheck->transaction = message();
+        pendingCheck->key = key;
+        pendingCheck->info = info;
+        m_authWait.append(pendingCheck);
+        return data;             // return value will be ignored
     }
 
     const AuthInfoContainer *result = findAuthInfoItem(key, info);
     if (!result || result->isCanceled)
     {
-       if (!result &&
-           (info.username.isEmpty() || info.password.isEmpty()) &&
-           !KWallet::Wallet::keyDoesNotExist(KWallet::Wallet::NetworkWallet(),
-                                             KWallet::Wallet::PasswordFolder(), makeWalletKey(key, info.realmValue)))
-       {
-          QMap<QString, QString> knownLogins;
-          if (openWallet(windowId)) {
-              if (readFromWallet(m_wallet, key, info.realmValue, info.username, info.password,
-                             info.readOnly, knownLogins))
-	      {
-		      info.setModified(true);
-                      // fall through
-	      }
-	  }
-       } else {
-           info.setModified(false);
-       }
-
-       QByteArray data2;
-       QDataStream stream(&data2, QIODevice::WriteOnly);
-       stream << info;
-       return data2;
+        if (!result &&
+            (info.username.isEmpty() || info.password.isEmpty()) &&
+            !KWallet::Wallet::keyDoesNotExist(KWallet::Wallet::NetworkWallet(),
+                                              KWallet::Wallet::PasswordFolder(),
+                                              makeWalletKey(key, info.realmValue)))
+        {
+            QMap<QString, QString> knownLogins;
+            if (openWallet(windowId)) {
+                if (readFromWallet(m_wallet, key, info.realmValue, info.username,
+                                   info.password, info.readOnly, knownLogins))
+                {
+                    info.setModified(true);
+                            // fall through
+                }
+            }
+        } else {
+            info.setModified(false);
+        }
+    } else {
+        updateAuthExpire(key, result, windowId, false);
+        info = copyAuthInfo(result);
     }
-
-    updateAuthExpire(key, result, windowId, false);
-
-    info = copyAuthInfo(result);
+    
     QByteArray data2;
-    QDataStream stream(&data2, QIODevice::WriteOnly);
-    stream << info;
+    QDataStream stream2(&data2, QIODevice::WriteOnly);
+    stream2 << info;
     return data2;
 }
 
+qlonglong KPasswdServer::checkAuthInfoAsync(KIO::AuthInfo info, qlonglong windowId,
+                                            qlonglong usertime)
+{
+    kDebug(130) << "User =" << info.username << ", WindowId =" << windowId << endl;
+    if (usertime != 0) {
+        kapp->updateUserTimestamp(usertime);
+    }
+
+    // send the request id back to the client
+    qlonglong requestId = getRequestId();
+    QDBusMessage reply(message().createReply(requestId));
+    QDBusConnection::sessionBus().send(reply);
+
+    // if the check depends on a pending query, delay it
+    // until that query is finished.
+    QString key = createCacheKey(info);
+    if (hasPendingQuery(key, info)) {
+        Request *pendingCheck = new Request;
+        pendingCheck->isAsync = true;
+        pendingCheck->requestId = requestId;
+        pendingCheck->key = key;
+        pendingCheck->info = info;
+        m_authWait.append(pendingCheck);
+        return 0; // ignored as we already sent a reply
+    }
+
+    const AuthInfoContainer *result = findAuthInfoItem(key, info);
+    if (!result || result->isCanceled)
+    {
+        if (!result &&
+            (info.username.isEmpty() || info.password.isEmpty()) &&
+            !KWallet::Wallet::keyDoesNotExist(KWallet::Wallet::NetworkWallet(),
+                                              KWallet::Wallet::PasswordFolder(),
+                                              makeWalletKey(key, info.realmValue)))
+        {
+            QMap<QString, QString> knownLogins;
+            if (openWallet(windowId)) {
+                if (readFromWallet(m_wallet, key, info.realmValue, info.username,
+                                   info.password, info.readOnly, knownLogins))
+                {
+                    info.setModified(true);
+                            // fall through
+                }
+            }
+        } else {
+            info.setModified(false);
+        }
+    } else {
+        updateAuthExpire(key, result, windowId, false);
+        info = copyAuthInfo(result);
+    }
+
+    emit checkAuthInfoAsyncResult(requestId, m_seqNr, info);
+    return 0; // ignored
+}
+
 QByteArray
-KPasswdServer::queryAuthInfo(const QByteArray &data, const QString &errorMsg, qlonglong windowId,
-                             qlonglong seqNr, qlonglong usertime, const QDBusMessage &msg)
+KPasswdServer::queryAuthInfo(const QByteArray &data, const QString &errorMsg,
+                             qlonglong windowId, qlonglong seqNr, qlonglong usertime)
 {
     KIO::AuthInfo info;
-    {
-        QByteArray data2(data);
-        QDataStream stream(&data2, QIODevice::ReadOnly);
-        stream >> info;
-    }
+    QDataStream stream(data);
+    stream >> info;
     kDebug(130) << "User =" << info.username << ", Message= " << info.prompt
                 << ", WindowId =" << windowId << endl;
     if ( !info.password.isEmpty() ) // should we really allow the caller to pre-fill the password?
@@ -267,8 +350,9 @@ KPasswdServer::queryAuthInfo(const QByteArray &data, const QString &errorMsg, ql
 
     QString key = createCacheKey(info);
     Request *request = new Request;
-    msg.setDelayedReply(true);
-    request->transaction = msg;
+    setDelayedReply(true);
+    request->isAsync = false;
+    request->transaction = message();
     request->key = key;
     request->info = info;
     request->windowId = windowId;
@@ -291,15 +375,63 @@ KPasswdServer::queryAuthInfo(const QByteArray &data, const QString &errorMsg, ql
     return QByteArray();        // return value is going to be ignored
 }
 
+qlonglong
+KPasswdServer::queryAuthInfoAsync(const KIO::AuthInfo &info, const QString &errorMsg,
+                                  qlonglong windowId, qlonglong seqNr, qlonglong usertime)
+{
+/*    KIO::AuthInfo info;
+    data >> info; */
+    kDebug(130) << "User =" << info.username << ", Message= " << info.prompt
+                << ", WindowId =" << windowId << endl;
+    if (!info.password.isEmpty()) {
+        kDebug(130) << "password was set by caller";
+    }
+    if (usertime != 0) {
+        kapp->updateUserTimestamp(usertime);
+    }
+
+    QString key = createCacheKey(info);
+    Request *request = new Request;
+    request->isAsync = true;
+    request->requestId = getRequestId();
+    request->key = key;
+    request->info = info;
+    request->windowId = windowId;
+    request->seqNr = seqNr;
+    if (errorMsg == "<NoAuthPrompt>") {
+        request->errorMsg.clear();
+        request->prompt = false;
+    } else {
+        request->errorMsg = errorMsg;
+        request->prompt = true;
+    }
+    m_authPending.append(request);
+
+    if (m_authPending.count() == 1) {
+        QTimer::singleShot(0, this, SLOT(processRequest()));
+    }
+
+    return request->requestId;
+}
+
+void
+KPasswdServer::addAuthInfo(const KIO::AuthInfo &info, qlonglong windowId)
+{
+    kDebug(130) << "User =" << info.username << ", RealmValue= " << info.realmValue
+                << ", WindowId = " << windowId << endl;
+    QString key = createCacheKey(info);
+
+    m_seqNr++;
+
+    addAuthInfoItem(key, info, windowId, m_seqNr, false);
+}    
+
 void
 KPasswdServer::addAuthInfo(const QByteArray &data, qlonglong windowId)
 {
     KIO::AuthInfo info;
-    {
-        QByteArray data2(data);
-        QDataStream stream(&data2, QIODevice::ReadOnly);
-        stream >> info;
-    }
+    QDataStream stream(data);
+    stream >> info;
     kDebug(130) << "User =" << info.username << ", RealmValue= " << info.realmValue
                 << ", WindowId = " << windowId << endl;
     QString key = createCacheKey(info);
@@ -330,7 +462,7 @@ KPasswdServer::removeAuthInfo(const QString& host, const QString& protocol, cons
                      << current->info.username;
             if (current->info.url.protocol() == protocol &&
                current->info.url.host() == host &&
-               current->info.username == user)
+               (current->info.username == user || user.isEmpty()))
             {
                 kDebug(130) << "Removing this entry";
                 removeAuthInfoItem(dictIterator.key(), current->info);
@@ -360,13 +492,20 @@ KPasswdServer::openWallet( int windowId )
 void
 KPasswdServer::processRequest()
 {
-    Request *request = m_authPending.first();
+    // guard so processRequest is only ever run once.
+    static bool processing = false;
+    if (processing) {
+        return;
+    }
+    processing = true;
+    
+    Request *request = m_authPending.takeFirst();
     if (!request)
        return;
 
     KIO::AuthInfo &info = request->info;
     bool bypassCacheAndKWallet = info.getExtraField(AUTHINFO_EXTRAFIELD_BYPASS_CACHE_AND_KWALLET).toBool() == true;
-    
+
     kDebug(130) << "User =" << info.username << ", Message =" << info.prompt << endl;
     const AuthInfoContainer *result = findAuthInfoItem(request->key, request->info);
 
@@ -417,7 +556,7 @@ KPasswdServer::processRequest()
 
             // assemble dialog-flags
             KPasswordDialog::KPasswordDialogFlags dialogFlags;
-            
+
             if (info.getExtraField(AUTHINFO_EXTRAFIELD_DOMAIN).isValid())
             {
                 dialogFlags |= KPasswordDialog::ShowDomainLine;
@@ -426,15 +565,15 @@ KPasswdServer::processRequest()
                     dialogFlags |= KPasswordDialog::DomainReadOnly;
                 }
             }
-            
-            if (info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).isValid()) 
+
+            if (info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).isValid())
             {
                 dialogFlags |= KPasswordDialog::ShowAnonymousLoginCheckBox;
             }
-            
-            dialogFlags |= (info.keepPassword ? ( KPasswordDialog::ShowUsernameLine 
+
+            dialogFlags |= (info.keepPassword ? ( KPasswordDialog::ShowUsernameLine
                   |  KPasswordDialog::ShowKeepPassword) : KPasswordDialog::ShowUsernameLine );
-            
+
             // instantiate dialog
             KPasswordDialog dlg( 0l, dialogFlags) ;
             dlg.setPrompt(info.prompt);
@@ -460,11 +599,11 @@ KPasswdServer::processRequest()
 
             if (info.getExtraField(AUTHINFO_EXTRAFIELD_DOMAIN).isValid ())
                 dlg.setDomain(info.getExtraField(AUTHINFO_EXTRAFIELD_DOMAIN).toString());
-            
+
             if (info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).isValid ())
                 dlg.setAnonymousMode(info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).toBool());
 
-#ifndef Q_WS_WIN            
+#ifndef Q_WS_WIN
             KWindowSystem::setMainWindow(&dlg, request->windowId);
 #else
             KWindowSystem::setMainWindow(&dlg, (HWND)(long)request->windowId);
@@ -477,7 +616,7 @@ KPasswdServer::processRequest()
                info.username = dlg.username();
                info.password = dlg.password();
                info.keepPassword = dlg.keepPassword();
-               
+
                if (info.getExtraField(AUTHINFO_EXTRAFIELD_DOMAIN).isValid ())
                    info.setExtraField(AUTHINFO_EXTRAFIELD_DOMAIN, dlg.domain());
                if (info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).isValid ())
@@ -514,67 +653,58 @@ KPasswdServer::processRequest()
         }
     }
 
-    QByteArray replyData;
-
-    QDataStream stream2(&replyData, QIODevice::WriteOnly);
-    stream2 << info;
-    QDBusConnection::sessionBus().send(request->transaction.createReply(QVariantList() << replyData << m_seqNr));
-
-    m_authPending.remove((unsigned int) 0);
-
-    // Check all requests in the wait queue.
-    for (Request *waitRequest = m_authWait.first(); waitRequest; )
-    {
-       bool keepQueued = false;
-       QString key = waitRequest->key;
-
-       request = m_authPending.first();
-       QString path2 = waitRequest->info.url.directory(KUrl::AppendTrailingSlash|KUrl::ObeyTrailingSlash);
-       for(; request; request = m_authPending.next())
-       {
-           if (request->key != key)
-               continue;
-
-           if (info.verifyPath)
-           {
-             QString path1 = request->info.url.directory(KUrl::AppendTrailingSlash|KUrl::ObeyTrailingSlash);
-               if (!path2.startsWith(path1))
-                   continue;
-           }
-
-           keepQueued = true;
-           break;
-       }
-       if (keepQueued)
-       {
-           waitRequest = m_authWait.next();
-       }
-       else
-       {
-           const AuthInfoContainer *result = findAuthInfoItem(waitRequest->key, waitRequest->info);
-           QByteArray replyData;
-
-           QDataStream stream2(&replyData, QIODevice::WriteOnly);
-
-           if (!result || result->isCanceled)
-           {
-               waitRequest->info.setModified(false);
-               stream2 << waitRequest->info;
-           }
-           else
-           {
-               updateAuthExpire(waitRequest->key, result, waitRequest->windowId, false);
-               KIO::AuthInfo info = copyAuthInfo(result);
-               stream2 << info;
-           }
-
-           QDBusConnection::sessionBus().send(waitRequest->transaction.createReply());
-
-           m_authWait.remove();
-           waitRequest = m_authWait.current();
-       }
+    if (request->isAsync) {
+        emit queryAuthInfoAsyncResult(request->requestId, m_seqNr, info);
+    } else {
+        QByteArray replyData;
+        QDataStream stream2(&replyData, QIODevice::WriteOnly);
+        stream2 << info;
+        QDBusConnection::sessionBus().send(request->transaction.createReply(QVariantList() << replyData << m_seqNr));
     }
 
+    delete request;
+
+    // Check all requests in the wait queue.
+    Request *waitRequest;
+    QMutableListIterator<Request*> it(m_authWait);
+    while (it.hasNext()) {
+        waitRequest = it.next();
+
+        if (!hasPendingQuery(waitRequest->key, waitRequest->info))
+        {
+            const AuthInfoContainer *result = findAuthInfoItem(waitRequest->key,
+                                                               waitRequest->info);
+            QByteArray replyData;
+
+            QDataStream stream2(&replyData, QIODevice::WriteOnly);
+
+            KIO::AuthInfo rcinfo;
+            if (!result || result->isCanceled)
+            {
+                waitRequest->info.setModified(false);
+                stream2 << waitRequest->info;
+            }
+            else
+            {
+                updateAuthExpire(waitRequest->key, result, waitRequest->windowId, false);
+                rcinfo = copyAuthInfo(result);
+                stream2 << rcinfo;
+            }
+
+            if (waitRequest->isAsync) {
+                emit checkAuthInfoAsyncResult(waitRequest->requestId, m_seqNr, rcinfo);
+            } else {
+                QDBusConnection::sessionBus().send(waitRequest->transaction.createReply(QVariantList() << replyData << m_seqNr));
+            }
+
+            delete waitRequest;
+            it.remove();
+        }
+    }
+
+    // reallow processing
+    processing = false;
+    
     if (m_authPending.count())
        QTimer::singleShot(0, this, SLOT(processRequest()));
 
